/*
 * File:	DemoGPIO.cpp
 *
 * Date:	2012-08-22
 * Company:	DH electronics GmbH
 * Author:	Ludwig Zenz, lzenz@dh-electronics.de
 *
 * Remark:	The following code aims to offer you some help with the
 * 			GPIO (General Purpose Input/Output) operations in Linux
 * 			user-space programs. We are using the gpio-sysfs.
 */


/*
 * For further GPIO documentation and examples (kernel-module gpio-calls) have a look at:
 * ->	http://www.kernel.org/doc/Documentation/gpio.txt
 */


/* IO Header */
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>

/* Mapping of DHCOM-gpios to CPU specific gpio numbers */
#include "am35xDHCOMgpios.h"
//#include "iMX25DHCOMgpios.h"

/* defines */
#define GPIO_DIR_OUTPUT	0
#define GPIO_DIR_INPUT 	1


/* prototypes */
int export_gpio_to_userspace(int gpio_nr);
int set_gpio_value(int gpio_nr, int value);
int get_gpio_value(int gpio_nr);
int set_gpio_direction(int gpio_nr, int direction);


int main()
{
	printf("\nStarting GPIO demo ...\n\n");

	printf("\n... exit with STRG+C \n\n");

	/*************************************************************************************
	 * export gpio a to userspace
	 */
	export_gpio_to_userspace(DHCOM_GPIO_A);


	/*************************************************************************************
	 * set direction of gpio
	 */
	set_gpio_direction(DHCOM_GPIO_A, GPIO_DIR_OUTPUT);


	/*************************************************************************************
	 * toggle gpio every second
	 */
	while (true)
	{

		if (get_gpio_value(DHCOM_GPIO_A))
			set_gpio_value(DHCOM_GPIO_A, 0);
		else
			set_gpio_value(DHCOM_GPIO_A, 1);

		sleep(1);
	}

return 0;
}


/************************************************************************************************
 * low level functions
 ************************************************************************************************/


/*
 * export a gpio to have access from user-space
 *
 * Brief:	To export an GPIO you need to open the export interface of the
 * 			gpio-sysfs and write the GPIO number you want to export.
 *
 * Remark:	Before manipulating the GPIO the specific GPIO pins
 * 			has to be exported and configured appropriately.
 */
int export_gpio_to_userspace(int gpio_nr)
{
	int file;
	char strGPIOnr[4];

	file = open("/sys/class/gpio/export", O_WRONLY);

	if (file < 0)
	{
		printf("ERROR opening gpio %d to export it!\n", gpio_nr);
		exit(EXIT_FAILURE); /* Terminate process. All open stdio streams are flushed and closed. */
	}

	snprintf(strGPIOnr, 4, "%d", gpio_nr);

	write(file, strGPIOnr, 4);

	close(file);

	return 0;
}


/*
 * set the gpio# value (0 = low / 1 = high)
 *
 * Brief:	Using the value interface of gpio-sysfs to set/clear the GPIO pin.
 */
int set_gpio_value(int gpio_nr, int value)
{
	int tmp;
	int file;
	char strGPIOvaluefile[30];

	snprintf(strGPIOvaluefile, 30, "/sys/class/gpio/gpio%d/value", gpio_nr);

	/* open value-file */
	file = open(strGPIOvaluefile, O_RDWR);
	if (file < 0)
	{
		printf("ERROR opening GPIO value for %d\n", gpio_nr);
		exit(EXIT_FAILURE); /* Terminate process. All open stdio streams are flushed and closed. */
	}

	/* write new value to value-file */
	if (value == 0)
		tmp = write(file, "0", 2);
	else
		tmp = write(file, "1", 2);

	/* check if the value was written properly */
	if (tmp != 2)
	{
		printf("ERROR setting GPIO value for %d\n", gpio_nr);
		exit(EXIT_FAILURE); /* Terminate process. All open stdio streams are flushed and closed. */
	}

	/* close value file */
	close(file);

	return 0;
}

/*
 * get the gpio# value (0 = low / 1 = high)
 *
 * Brief:	Using the value interface of gpio-sysfs to set/clear the GPIO pin.
 *
 * Return:	0 if gpio-value is 0 (low) / 1 if gpio-value is 1 (high)
 */
int get_gpio_value(int gpio_nr)
{
	int file;
	char strGPIOvaluefile[30];
	char strValue[2];

	snprintf(strGPIOvaluefile, 30, "/sys/class/gpio/gpio%d/value", gpio_nr);

	/* open value-file */
	file = open(strGPIOvaluefile, O_RDWR);
	if (file < 0)
	{
		printf("ERROR opening GPIO value for %d\n", gpio_nr);
		exit(EXIT_FAILURE); /* Terminate process. All open stdio streams are flushed and closed. */
	}

	/* read value */
	if (read(file, strValue, 2) < 0)
	{
		printf("ERROR reading GPIO value of %d\n", gpio_nr);
		exit(EXIT_FAILURE); /* Terminate process. All open stdio streams are flushed and closed. */
	}

	/* close value file */
	close(file);

	if (strValue[0] == '0')
		return 0;

	if (strValue[0] == '1')
		return 1;

	printf("ERROR unexpected GPIO value of %d\n", gpio_nr);
	return -1;
}

/*
 * get the gpio# value (0 = low / 1 = high)
 *
 * Brief:	Using the value interface of gpio-sysfs to set/clear the GPIO pin.
 *
 * Return:	0 if gpio-value is 0 (low) / 1 if gpio-value is 1 (high)
 */
int set_gpio_direction(int gpio_nr, int direction)
{
	int tmp;
	int file;
	char strGPIOdirectionfile[34];

	snprintf(strGPIOdirectionfile, 34, "/sys/class/gpio/gpio%d/direction", gpio_nr);

	/* open direction-file */
	file = open(strGPIOdirectionfile, O_RDWR);
	if (file < 0)
	{
		printf("ERROR opening GPIO direction for %d\n", gpio_nr);
		exit(EXIT_FAILURE); /* Terminate process. All open stdio streams are flushed and closed. */
	}

	if (direction == 0)
		tmp = write(file, "out", 4);
	else
		tmp = write(file, "in", 3);

	/* close direction-file */
	close(file);

	/* check if the direction was written properly */
	if (tmp < 3)
	{
		printf("ERROR setting GPIO direction for %d\n", gpio_nr);
		exit(EXIT_FAILURE); /* Terminate process. All open stdio streams are flushed and closed. */
	}

	return 0;
}



